@@ -4,6 +4,7 @@
 import os
 import socket
 import struct
+import time
 import zlib
 from Crypto.Cipher import DES
 from contextlib import contextmanager
@@ -115,20 +116,28 @@ class NetDimm:
             self.version = info.firmware_version
             return info
 
-    def send(self, data: Union[bytes, FileBytes], key: Optional[bytes] = None, disable_crc_check: bool = False, progress_callback: Optional[Callable[[int, int], None]] = None) -> None:
+    def send(
+        self,
+        data: Union[bytes, FileBytes],
+        key: Optional[bytes] = None,
+        disable_crc_check: bool = False,
+        disable_now_loading: bool = False,
+        progress_callback: Optional[Callable[[int, int], None]] = None,
+    ) -> None:
         with self.connection():
             # First, signal back to calling code that we've started
             if progress_callback:
                 progress_callback(0, len(data))
 
-            # Reboot and display "now loading..." on the cabinet screen
-            self.__set_host_mode(1)
+            if not disable_now_loading:
+                # Reboot and display "now loading..." on the cabinet screen
+                self.__set_host_mode(1)
 
-            # The official sega transfer tool update sleeps for 5 seconds
-            # here, presumably because the net dimm doesn't respond to
-            # packets while it is ressetting the target to display "now
-            # loading". However, our timeout is 10 seconds so this ends
-            # up working without that sleep in practice.
+                # The official sega transfer tool update sleeps for 5 seconds
+                # here, presumably because the net dimm doesn't respond to
+                # packets while it is ressetting the target to display "now
+                # loading". However, our timeout is 10 seconds so this ends
+                # up working without that sleep in practice.
 
             if key:
                 # Send the key that we're going to use to encrypt
@@ -205,16 +214,22 @@ class NetDimm:
         if self.sock is None:
             raise NetDimmException("Not connected to NetDimm")
 
-        # a function to receive a number of bytes with hard blocking
-        res: List[bytes] = []
-        left: int = num
+        try:
+            # a function to receive a number of bytes with hard blocking
+            res: List[bytes] = []
+            left: int = num
+            start = time.time()
 
-        while left > 0:
-            ret = self.sock.recv(left)
-            left -= len(ret)
-            res.append(ret)
+            while left > 0:
+                if time.time() - start > 10.0:
+                    raise NetDimmException("Could not receive data from NetDimm")
+                ret = self.sock.recv(left)
+                left -= len(ret)
+                res.append(ret)
 
-        return b"".join(res)
+            return b"".join(res)
+        except Exception as e:
+            raise NetDimmException("Could not receive data from NetDimm") from e
 
     @contextmanager
     def connection(self) -> Generator[None, None, None]:
@@ -263,16 +278,19 @@ class NetDimm:
     def __send_packet(self, packet: NetDimmPacket) -> None:
         if self.sock is None:
             raise NetDimmException("Not connected to NetDimm")
-        self.sock.send(
-            struct.pack(
-                "<I",
-                (
-                    ((packet.pktid & 0xFF) << 24) |  # noqa: W504
-                    ((packet.flags & 0xFF) << 16) |  # noqa: W504
-                    (packet.length & 0xFFFF)
-                ),
-            ) + packet.data
-        )
+        try:
+            self.sock.send(
+                struct.pack(
+                    "<I",
+                    (
+                        ((packet.pktid & 0xFF) << 24) |  # noqa: W504
+                        ((packet.flags & 0xFF) << 16) |  # noqa: W504
+                        (packet.length & 0xFFFF)
+                    ),
+                ) + packet.data
+            )
+        except Exception as e:
+            raise NetDimmException("Could not send data to NetDimm") from e
 
     def __recv_packet(self) -> NetDimmPacket:
         # First read the header to get the packet length.
@@ -585,6 +603,8 @@ class NetDimm:
             addr += curlen
             sequence += 1
 
+        if progress_callback:
+            progress_callback(addr, total)
         crc = (~crc) & 0xFFFFFFFF
         self.__print("length: %08x" % addr)
         self.__set_information(crc, addr)
